import PyLibrary.funciones_comunes as fc
import pandas as pd
import os
from translation_config import TranslationConfig
from openpyxl import load_workbook
from openpyxl.styles import Font
from openpyxl.comments import Comment

# Definir el logger a nivel de módulo
logger = None


def importar_traduccion_manual(config: TranslationConfig):
    archivo_maestro = config.get_master_path()
    if not os.path.exists(archivo_maestro):
        print("El archivo maestro no existe.")
        return

    archivo_traduccion = config.get_translate_path()
    master_col = config.codigo_idioma_seleccionado
    propuesto_col = f"{config.codigo_idioma_seleccionado}_Propuesto"

    df_maestro = fc.read_dataframe_with_cleanup_retries(archivo_maestro)
    df_traduccion = fc.read_dataframe_with_cleanup_retries(archivo_traduccion)

    celdas_modificadas = {}
    celdas_con_errores = {}

    for index, fila in df_traduccion.iterrows():
        clave = fila[df_maestro.columns[0]]
        if clave in df_maestro[df_maestro.columns[0]].values:
            valor_traducido = fila[propuesto_col]  # Use propuesto column
            valor_original = df_maestro.loc[
                df_maestro[df_maestro.columns[0]] == clave, master_col
            ].values[0]

            if (
                pd.notnull(valor_traducido)
                and valor_traducido != ""
                and str(valor_original) != str(valor_traducido)
            ):

                okToSave, Error = fc.verificar_celda_traducida(
                    config.codigo_tipo_PLC, clave, valor_traducido
                )
                if okToSave:
                    df_maestro.loc[
                        df_maestro[df_maestro.columns[0]] == clave, master_col
                    ] = valor_traducido
                    logger.info(
                        f'Fila {index}, Columna {master_col}: "{valor_original}" actualizado a "{valor_traducido}"'
                    )

                    fila_excel = (
                        df_maestro.index[
                            df_maestro[df_maestro.columns[0]] == clave
                        ].tolist()[0]
                        + 2
                    )
                    columna_excel = df_maestro.columns.get_loc(master_col) + 1
                    celdas_modificadas[(fila_excel, columna_excel)] = valor_traducido
                else:
                    logger.error(
                        f"Fila {index}, Columna {master_col}: No actualizado porque: {Error}"
                    )
                    fila_excel = index + 2
                    columna_excel = df_traduccion.columns.get_loc(propuesto_col) + 1
                    celdas_con_errores[(fila_excel, columna_excel)] = Error

    fc.save_dataframe_with_retries(df_maestro, output_path=archivo_maestro)

    aplicar_formato_celdas_maestro(archivo_maestro, celdas_modificadas)

    marcar_errores_archivo_fuente(archivo_traduccion, celdas_con_errores)

    print(
        f"Traducciones manuales importadas y archivo maestro actualizado: {archivo_maestro}."
    )
    print(
        f"Se han actualizado {len(celdas_modificadas)} celdas en el maestro y se han encontrado {len(celdas_con_errores)} errores en el archivo fuente."
    )


def aplicar_formato_celdas_maestro(archivo, celdas_modificadas):
    workbook = load_workbook(archivo)
    sheet = workbook.active

    for (fila, columna), valor in celdas_modificadas.items():
        celda = sheet.cell(row=fila, column=columna)
        celda.font = Font(bold=True)
        celda.value = valor

    workbook.save(archivo)
    print(
        f"Se han formateado {len(celdas_modificadas)} celdas modificadas en negrita en el archivo maestro."
    )


def marcar_errores_archivo_fuente(archivo, celdas_con_errores):
    workbook = load_workbook(archivo)
    sheet = workbook.active

    for (fila, columna), mensaje_error in celdas_con_errores.items():
        celda = sheet.cell(row=fila, column=columna)
        celda.font = Font(color="FF0000")  # Texto en rojo
        comment = Comment(mensaje_error, "Sistema de Traducción")
        celda.comment = comment

    workbook.save(archivo)
    print(
        f"Se han marcado {len(celdas_con_errores)} celdas con errores en rojo en el archivo fuente."
    )


def run(config: TranslationConfig):
    global logger
    logger = fc.configurar_logger(config.work_dir)
    script_name = os.path.basename(__file__)
    print(f"\rIniciando: {script_name}\r")
    importar_traduccion_manual(config)


if __name__ == "__main__":
    import menu_pasos_traduccion

    menu_pasos_traduccion.main()
