import PyLibrary.funciones_comunes as fc
import pandas as pd
import os
from translation_config import TranslationConfig
from openpyxl import load_workbook
from openpyxl.styles import Font
from openpyxl.styles import PatternFill, Alignment, Font

# Definir el logger a nivel de módulo
logger = None


def importar_traduccion(config: TranslationConfig):
    archivo_maestro = config.get_master_path()
    if not os.path.exists(archivo_maestro):
        print("El archivo maestro no existe.")
        return

    master_col = config.codigo_idioma_seleccionado
    master_propuesto_col = f"{master_col}_Propuesto"
    translated_col = f"{config.codigo_idioma_seleccionado} Translated"
    affinity_col = f"{config.codigo_idioma_seleccionado} Affinity"

    df_maestro = fc.read_dataframe_with_cleanup_retries(archivo_maestro)
    df_traduccion = fc.read_dataframe_with_cleanup_retries(
        config.get_auto_translate_path()
    )

    # Caso especial: columna maestra es igual al idioma seleccionado
    is_same_column = config.codigo_columna_maestra == config.codigo_idioma_seleccionado

    # Si es el mismo, asegurarse que existe la columna propuesta
    if is_same_column and master_propuesto_col not in df_maestro.columns:
        df_maestro[master_propuesto_col] = df_maestro[master_col]

    celdas_modificadas = {}

    if config.codigo_columna_maestra not in df_maestro.columns:
        print(f"Error: Columnas requeridas no encontradas en el archivo maestro")
        return

    if (
        config.codigo_columna_maestra not in df_traduccion.columns
        or translated_col not in df_traduccion.columns
        or affinity_col not in df_traduccion.columns
    ):
        print(f"Error: Columnas requeridas no encontradas en el archivo de traducción")
        return

    for index, fila in df_traduccion.iterrows():
        clave = fila[config.codigo_columna_maestra]
        mascara = df_maestro[config.codigo_columna_maestra] == clave

        if mascara.any():
            if (
                pd.notna(fila[affinity_col])
                and fila[affinity_col] >= config.nivel_afinidad_minimo
                and pd.notna(fila[translated_col])
            ):
                valor_traducido = fila[translated_col]
                indice_maestro = df_maestro.index[mascara].tolist()[0]

                # Determinar la columna a actualizar según el caso
                columna_destino = master_propuesto_col if is_same_column else master_col
                valor_original = df_maestro.loc[indice_maestro, columna_destino]

                if str(valor_original) != str(valor_traducido):
                    df_maestro.loc[indice_maestro, columna_destino] = valor_traducido
                    logger.info(
                        f'Fila {index}, Columna {columna_destino}: "{valor_original}" actualizado a "{valor_traducido}"'
                    )

                    fila_excel = indice_maestro + 2
                    columna_excel = df_maestro.columns.get_loc(columna_destino) + 1
                    celdas_modificadas[(fila_excel, columna_excel)] = valor_traducido

    # Guardar con formato Excel
    with pd.ExcelWriter(archivo_maestro, engine="openpyxl") as writer:
        df_maestro.to_excel(writer, index=False, sheet_name="Sheet1")

        workbook = writer.book
        worksheet = writer.sheets["Sheet1"]

        # Inmovilizar paneles en A2
        worksheet.freeze_panes = "A2"

        # Configurar ancho de columnas basado en contenido
        from openpyxl.utils import get_column_letter
        from openpyxl.styles import Alignment, Font, PatternFill

        for col in worksheet.columns:
            max_length = 0
            column = col[0].column_letter

            for cell in col:
                try:
                    if cell.value:
                        text_length = len(str(cell.value))
                        if text_length > 50:
                            cell.alignment = Alignment(wrap_text=True, vertical="top")
                            text_length = min(
                                50, max(len(word) for word in str(cell.value).split())
                            )
                        max_length = max(max_length, text_length)
                except:
                    pass

            adjusted_width = min(50, max_length + 2)
            worksheet.column_dimensions[column].width = (
                adjusted_width if adjusted_width > 8 else 8
            )

        # Aplicar negrita y color a las celdas modificadas
        light_blue = PatternFill(
            start_color="ADD8E6", end_color="ADD8E6", fill_type="solid"
        )
        for (fila, columna), valor in celdas_modificadas.items():
            celda = worksheet.cell(row=fila, column=columna)
            celda.font = Font(bold=True)
            celda.value = valor
            celda.fill = light_blue

    if celdas_modificadas:
        print(
            f"Se han actualizado y marcado {len(celdas_modificadas)} celdas en el archivo maestro."
        )
    else:
        print("No se realizaron modificaciones en el archivo maestro.")


def aplicar_negrita_celdas_modificadas(archivo, celdas_modificadas):
    workbook = load_workbook(archivo)
    sheet = workbook.active

    for (fila, columna), valor in celdas_modificadas.items():
        celda = sheet.cell(row=fila, column=columna)
        celda.font = Font(bold=True)
        celda.value = valor

    workbook.save(archivo)
    print(
        f"Se han marcado en negrita {len(celdas_modificadas)} celdas modificadas en {archivo}."
    )


def run(config: TranslationConfig):
    global logger
    logger = fc.configurar_logger(config.work_dir)
    script_name = os.path.basename(__file__)
    print(f"\rIniciando: {script_name}\r")
    importar_traduccion(config)


if __name__ == "__main__":
    import menu_pasos_traduccion

    menu_pasos_traduccion.main()
